package  miss.misdata;

import   rsf.vector.*;
import   rsf.solver.*;
import   sep.io.ArgumentKeeper;
import   conv.convt.*;
import  juice.solver.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class MissingData {
  public static void main(String[] args) {
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);

    String  inRsf  = argKeeper.getString("in",    "../Data/hole.HH");
    String outRsf  = argKeeper.getString("out",   "fill.H");
    String filtRsf = argKeeper.getString("filt",  "");
    int    niter   = argKeeper.getInt   ("niter", "30");
    float  tol     = argKeeper.getFloat ("tol",   "1.0e-13F");

    Rsf filter =
      filtRsf.equals("") ? (rsf.util.Laplacian.newRsf(2)):(RsfFactory.newRsf(filtRsf));
    Rsf holyData = RsfFactory.newRsf(inRsf);
    RsfSpace dataSpace = (RsfSpace) holyData.getSpace();
    Tcai F = new TcaiFactory(filter).getTcaiFromInputSpace(dataSpace);
    
    Rsf infillData = RsfFactory.newRsf(dataSpace);
    infillData.zero();

    MaskedEqns mdeq  = new MaskedEqns(F, holyData);
    mdeq.zeroKnownComponents(infillData);
    NormalEqns nreq  = new NormalEqns(mdeq.getAM(), mdeq.getNegAk());
    CGSolver        cgSlv = new CGSolver(new IteratorSimple(tol, niter, false));
      
    cgSlv.solve(nreq.getAtA(), nreq.getAtb(), infillData);

    infillData.add(holyData);
    infillData.write(outRsf);
  }

  /** returns a laplacian filter */
  /*
  static Rsf getLaplacian() {
    Rsf2 filter = (Rsf2) RsfFactory.newRsf(3,3);
    filter.fill(0);
    filter.setDatum(1,1, 4F);
    filter.setDatum(0,1,-1F);
    filter.setDatum(2,1,-1F);
    filter.setDatum(1,0,-1F);
    filter.setDatum(1,2,-1F);
    return filter;
  }
  */
  /** returns the correct filter for the data hole.H (from tdf) */
  static Rsf getCorrectFilter() {
    Rsf2 filter = (Rsf2) RsfFactory.newRsf(5,3);
    filter.fill(0);
    filter.setDatum(0,2,  1F);
    filter.setDatum(2,3,  1F);
    filter.setDatum(1,1, -1F);
    filter.setDatum(1,4, -1F);
    return filter;
  }
}
