package  miss.misfilt;

import   rsf.vector.*;
import   rsf.solver.*;
import   conv.convi.*;
import  juice.operator.CompoundLinOpAdj;
import  juice.solver.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class MissingFilter {
  public static void main(String[] args) {
    Rsf holyData = RsfFactory.newRsf("../Data/hole.HH");
    RsfSpace pefSpace = new RsfSpace(5, 3);
    InternalConvolution conv =
      new InternalConvolution(InternalConvolution.kInput,
			      (RsfSpace) holyData.getSpace(),
			      InternalConvolution.kFilter,
			      pefSpace                      );
    Rsf pefNonZeroMask  = getPefNonZeroMask (pefSpace);
    Rsf dataSupportCmpl = getDataSupportCmpl(holyData);

    Mask influenceMaskOp =
      new SepMask((Rsf) conv.imageBinary(dataSupportCmpl, pefNonZeroMask),
		  true);
    Icaf holyDataIcaf = conv.getIcaf(holyData);

    // the order of operators is left to right, so holyDataIcaf will be first
    CompoundLinOpAdj A  = new CompoundLinOpAdj(holyDataIcaf, influenceMaskOp);
    Rsf pefKnown        = getPefKnown(pefSpace);
    Rsf pefFreeMask     = getPefFreeMask(pefNonZeroMask);

    Rsf pefSolution = RsfFactory.newRsf(pefSpace);

    Mask pefFreeMaskOp  = new SepMask(pefFreeMask);

    System.out.println("Solving for the filter\n");

    MaskedEqns mdeq  = new MaskedEqns(A, pefKnown,
							  pefFreeMaskOp);
    mdeq.zeroKnownComponents(pefSolution);
    NormalEqns nreq  = new NormalEqns(mdeq.getAM(), mdeq.getNegAk());
    CGSolver   cgSlv = new CGSolver(new IteratorSimple((float) 1.0e-13,
							    20, true));
      
    cgSlv.solve(nreq.getAtA(), nreq.getAtb(), pefSolution);

    pefSolution.add(pefKnown);
    pefSolution.write("filter.H");

    System.out.println("\n\n\nSolving for the data\n");

    String[] misDataArgs = {"filt=filter.H"};
    miss.misdata.MissingData.main(misDataArgs);
  }

  static Rsf getPefFreeMask(Rsf pefNonZeroMask) {
    Rsf2 pefFreeMask = (Rsf2) pefNonZeroMask.copy();
    pefFreeMask.setDatum(0, 2, 0F);
    return pefFreeMask;
  }

  static Rsf getPefKnown(RsfSpace pefSpace) {
    Rsf2 pefKnown = (Rsf2) RsfFactory.newRsf(pefSpace);
    pefKnown.fill(0F);
    pefKnown.setDatum(0, 2, 1F);
    return pefKnown;
  }

  static Rsf getPefNonZeroMask(RsfSpace pefSpace) {
    Rsf2 pefMask = (Rsf2) pefSpace.newMember();
    pefMask.fill(1F);
    pefMask.setDatum(0,0, 0F);
    pefMask.setDatum(0,1, 0F);
    return pefMask;
  }

  static Rsf getDataSupportCmpl(Rsf holyData) {
    SupportOp C = new SupportOp((RsfSpace) holyData.getSpace()).getComplement();
    return (Rsf) C.image(holyData);
  }
}
