package miss.misfiltF;

import   sep.operator.SepOperator;
import   jam.operator.*;
import   jam.vector.*;
import   jam.JamObject;
import  rsfF.vector.*;

/**
 * A <TT>SupportOp</TT> is an operator from an <TT>RsfSpace</TT> to itself
 * in which the output is zero whereever the input is zero and one wherever
 * the input is non-zero.<BR>
 *
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class SupportOp extends SepOperator {
  protected  SupportOp complement = null;
  protected  boolean   isCmp      = false;

  public     SupportOp(RsfSpace domainAndRange) { super(domainAndRange); }

  /** copy constructor */
  protected  SupportOp(SupportOp src) {
    super(src);
    this.isCmp = src.isCmp;
  }

  void createComplement() {
    try {
      this.complement            =  (SupportOp) this.clone();
    }
    catch (Exception e) { System.err.println(e); }
    this.complement.isCmp      = !this.isCmp;
    this.complement.complement =  this;
  }

  /**
   * The complement of a <TT>SupportOp</TT>, S, is the operator, C, which returns
   * the complement of the support of the <TT>Rsf</TT>.<BR>
   * Thus, whereas, S(x) is 0 where x is zero and 1 where x is non-zero,
   * C(x) is 1 where x is zero and 0 where x is non-zero.
   */
  public SupportOp getComplement() {
    if (complement == null) createComplement();
    return complement;
  }

  protected void apply(boolean add, Vector dom, Vector ran) {
    if (!add) ran.zero();

    float[] x = ((Rsf) dom).d;
    float[] y = ((Rsf) ran).d;

    if (!isCmp) {
      for (int i = 0; i < x.length; i++)
	if (x[i] != 0F)
	  y[i] += 1F;
    }
    else {
      for (int i = 0; i < x.length; i++)
	if (x[i] == 0F)
	  y[i] += 1F;
    }
  }

  public JamObject copy() { return new SupportOp(this); }
}
