package  miss.seabeamF;

import   rsfF.vector.*;
import   rsfF.solver.*;
import   conv.conviF.*;
import   miss.misfiltF.SupportOp;
import  juice.operator.CompoundLinOpAdj;
import  juice.solver.*;
import  sep.io.ArgumentKeeper;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class Infill {
  public static void main(String[] args) {
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);

    int    niterd = argKeeper.getInt   ("niterd", "100");
    int    niterf = argKeeper.getInt   ("niterf",  "18");
    String inName = argKeeper.getString("in");

    Rsf inData = new Rsf(inName);

    RsfSpace pefSpace = new RsfSpace(5, 3);
    InternalConvolution conv =
      new InternalConvolution(InternalConvolution.kInput,
			      (RsfSpace) inData.getSpace(),
			      InternalConvolution.kFilter,
			      pefSpace                      );
    Rsf pefNonZeroMask  = getPefNonZeroMask (pefSpace);
    Rsf dataSupportCmpl = getDataSupportCmpl(inData);

    Mask influenceMaskOp =
      new SepMask((Rsf) conv.imageBinary(dataSupportCmpl, pefNonZeroMask))
      .getComplement();
    Icaf inDataIcaf = conv.getIcaf(inData);

    // the order of operators is left to right, so inDataIcaf will be first
    CompoundLinOpAdj A  = new CompoundLinOpAdj(inDataIcaf, influenceMaskOp);
    Rsf pefKnown        = getPefKnown(pefSpace);
    Rsf pefFreeMask     = getPefFreeMask(pefNonZeroMask);

    Rsf pefSolution = new Rsf(pefSpace);
    Mask pefFreeMaskOp  = new SepMask(pefFreeMask);

    System.out.println("Solving for the filter\n");

    MaskedEqns mdeq  = new MaskedEqns(A, pefKnown, pefFreeMaskOp);
    mdeq.zeroKnownComponents(pefSolution);
    NormalEqns nreq  = new NormalEqns(mdeq.getAM(), mdeq.getNegAk());
    CGSolver        cgSlv = new CGSolver(new IteratorSimple((float) 1.0e-50,
							    niterf, true));
      
    cgSlv.solve(nreq.getAtA(), nreq.getAtb(), pefSolution);
    pefSolution.add(pefKnown);

    pefSolution.write("filter.H");

    System.out.println("\n\n\nSolving for the data\n");

    String[] misDataArgs = {"in="   + inName,
			    "filt=" + "filter.H",
			    "niter="+ niterd,
			    "tol="  + "1e-50F"};
    miss.misdata.MissingData.main(misDataArgs);
  }

  static Rsf getPefFreeMask(Rsf pefNonZeroMask) {
    Rsf pefFreeMask = (Rsf) pefNonZeroMask.copy();
    pefFreeMask.setDatum(2, 0, 0F);
    return pefFreeMask;
  }

  static Rsf getPefKnown(RsfSpace pefSpace) {
    Rsf pefKnown = new Rsf(pefSpace);
    pefKnown.fill(0F);
    pefKnown.setDatum(2, 0, 1F);
    return pefKnown;
  }

  static Rsf getPefNonZeroMask(RsfSpace pefSpace) {
    Rsf pefMask = (Rsf) pefSpace.newMember();
    pefMask.fill(1F);
    pefMask.setDatum(0,0, 0F);
    pefMask.setDatum(1,0, 0F);
    return pefMask;
  }

  static Rsf getDataSupportCmpl(Rsf inData) {
    SupportOp C = new SupportOp((RsfSpace) inData.getSpace()).getComplement();
    return (Rsf) C.image(inData);
  }
}
